package jp.gr.java_conf.skrb.game.pong;

public class DemoState implements State {
    private final static int MAX_BALL = 3;
    private final static int SCORE_INC = 10;

    private PongGameEngine engine;
    private PadController padController;
    private BallController ballController;
    private PongView view;

    private int numBall;
    private int score;
    private int highScore;
    private boolean gaming = false;
    private boolean quiting = false;

    public DemoState(PongGameEngine engine, 
		  PongView view,
		  PadController padController,
		  BallController ballController) {
	this.engine = engine;
	this.view = view;
	this.padController = padController;
	this.ballController = ballController;
    }

    public State execute() {
	InputListener listener = new InputListener() {
		public void gameStart(InputEvent event) {
		    start();
		}

		public void gameQuit(InputEvent event) {
		    quit();
		}

		public void padMoving(InputEvent event) {}
	    };

	view.addInputListener(listener);

	highScore = engine.getHighScore();
	
	padController.init();

	while (!isGaming() && !isQuit()) {
	    score = 0;
	    for (numBall = 1; numBall <=  MAX_BALL; numBall++) {
		init();
		play();
		if (isGaming() || isQuit()) {
		    break;
		}
	    }
	}

	view.removeInputListener(listener);

	if (isGaming()) {
	    return new GameState(engine, view, padController, ballController);
	} else {
	    return null;
	}
    }

    private synchronized void start() {
	gaming = true;
    }

    private synchronized boolean isGaming() {
	return gaming;
    }

    private synchronized void quit() {
	quiting = true;
    }

    private synchronized boolean isQuit() {
	return quiting;
    }

    private void init() {
        ballController.init();
	
	view.draw(MAX_BALL - numBall, score, highScore,
		  ballController.getBall(), padController.getPad(), false);
    }

    private void play() {
	try {
	    while (!isGaming() && !isQuit()) {
		Thread.sleep(40L);

		boolean isSurviving = ballController.update();
		updatePad(ballController.getBall().getX());

		if (ballController.bounceCheck(padController.getBounds())) {
		    score += SCORE_INC;
		}

		view.draw(MAX_BALL - numBall, score, highScore,
			  ballController.getBall(), padController.getPad(), false);

		if (!isSurviving) {
		    break;
		}
	    }
	} catch (InterruptedException ex) {
	    ex.printStackTrace();
	}
    }

    private void updatePad(int x) {
	int padX = padController.getX();
	int width = padController.getWidth();
	int step = padController.getStep();
	int center = padX + width / 2;

	if (x > center + step) {
	    padController.moveRight();
	} else if (x < center - step) {
	    padController.moveLeft();
	}
    }
}
